#ifndef CHUMP_H
#define CHUMP_H

#include <glib.h>

#define uchar unsigned char

#define DEFINITIONSTRING "define"
#define INTIGERSTRING "int"
#define RELATIVESTRING "relative"
#define UINTIGERSTRING "uint"
#define URELATIVESTRING "urelative"
#define SYMBOLSTRING "symbol"
#define ENUMERATESTRING "enum"



typedef enum {NONTERM, TERM, TERM2FREE, NONTERM2FREE, SPLITNONTERM, SPLITNONTERM2FREE, SYMBOL, SYMBOL2FREE} RuleElementType;
typedef enum {TRANRULE, INTRULE, RELATIVERULE, UINTRULE, URELATIVERULE, SYMBOLRULE} RuleType; 
typedef enum {SPLIT, SPLIT2FREE} SplitType;



typedef struct DefinitionStackName{
 struct DefinitionStackName* next;
 char* string;
 GList* rules;
 } DefinitionStack;

typedef struct {
 char* string;
 RuleElementType type; 
 int twin;
 int twinsplit;
 GList* split;
 GList* rules;
 } RuleElement;
 
 
 
typedef struct {
 char* string;
 GList* bin;
 int twin;
 int size;
 SplitType type;
 } SplitElement;
 

typedef struct {
 RuleType type;
 GList* asci;
 GList* bit;
 } Rule;

typedef struct {
 GList* distext;
 GList* bitfield;
} Disasm;

typedef struct {
 GList* binary;
 char*  text;
 GList* ruletext;
} Asm;

typedef struct {
long value;
char *name;
int flags;
} Symbol;


typedef struct {
Symbol* symbol_table;
int symbol_count;
long address;
} AsmStatus;




GList dis_error;
const AsmStatus asm_default_status;

DefinitionStack* asm_parser (GList*);
DefinitionStack* asm_define (GList*, DefinitionStack*, char*);
Rule* 			 asm_rule   (GList*, DefinitionStack*);

Disasm asm_disassemble_number (GList* bitfield, GList * rules, AsmStatus);
Disasm asm_disassemble_symbol (GList* bitfield, GList * rules, AsmStatus);
Disasm asm_disassemble(GList* bitfield,GList* rules,AsmStatus);

Asm* asm_assemble_number (char* text, GList* rules, AsmStatus, int last);
Asm* asm_assemble_symbol (char* text, GList* rules, AsmStatus, int last);
GList* asm_assemble(char*,GList* rules,AsmStatus,int);

Asm asm_assembleold(char*,GList* rules,int,int,int);
GList* uchararr2glist(int, uchar*);
void asm_print(GList*);
GList* asm_print_symbols (GList* list, GList* replylist);
char* asm_sprint (GList* list, char*);
void asm_freestringlist(GList*);
GList* asm_copystringlist(GList*);
char* asm_sprint_tabstring (char*,char*);
int asm_sbprint (GList* list, uchar** data);

char* asm_symboltable_find_address (Symbol* symbol_table, int symbol_count,long address);
int asm_getsymbol_length (char* c);
int asm_findstrinsymboltable(char* string, Symbol* array,int arrcount);

#endif
